@@ -86,7 +86,8 @@ GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 	static const SpecialAnswersStruct SpecialAnswers[] = {
 		/* Standard GSM */
 		{"+CGREG:"	,1, ID_GetNetworkInfo},
-		{"+CBM:"	,1, ID_All},
+		/* Following has 2 lines in PDU mode, 1 line in TEXT ... */
+		{"+CBM:"	,2, ID_All},
 		{"+CMT:"	,2, ID_All},
 		{"+CMTI:"	,1, ID_All},
 		{"+CDS:"	,2, ID_All},
@@ -145,6 +146,9 @@ GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 		/* ONDA */
 		{"+ZUSIMR:"	,1, ID_All}, /* +ZUSIMR:2 */
 
+		/* Telit */
+		{"#STN:"	,1, ID_All}, /* #STN: 150,1,"" */
+
 		{NULL		,1, ID_All}};
 
 	/* We're starting new message */
@@ -286,6 +290,7 @@ GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 			d->Msg.Length			= 0;
 			break;
 		}
+		FALLTHROUGH;
 	default:
 		if (d->wascrlf) {
 			d->LineStart	= d->Msg.Length - 1;
