@@ -17,7 +17,7 @@
 
 #define SMSD_SHM_VERSION (2)
 #define SMSD_SHM_KEY (0xfa << 16 || SMSD_SHM_VERSION)
-#define SMSD_DB_VERSION (16)
+#define SMSD_DB_VERSION (17)
 
 #include "log.h"
 
@@ -87,6 +87,7 @@ struct _GSM_SMSDConfig {
 	const char   *RunOnReceive;
 	const char   *RunOnFailure; /* run this command on phone communication failure */
 	const char   *RunOnSent; /* run this command when an SMS has been sent successfully */
+	const char   *RunOnIncomingCall; /* run this command when a phone call has been canceled */
 	gboolean checksecurity;
 	gboolean hangupcalls;
 	gboolean checkbattery;
@@ -109,6 +110,7 @@ struct _GSM_SMSDConfig {
 	GSM_SMSC	SMSC, SMSCCache;
 	const char	*skipsmscnumber;
 	int		IgnoredMessages;
+	gboolean 	SkipMessage[GSM_MAX_MULTI_SMS];
 
 #if defined(HAVE_MYSQL_MYSQL_H) || defined(HAVE_POSTGRESQL_LIBPQ_FE_H) || defined(LIBDBI_FOUND) || defined(ODBC_FOUND)
 	/* options for SQL database */
@@ -195,6 +197,8 @@ struct _GSM_SMSDConfig {
 	 * Message reference set by callback from libGammu.
 	 */
 	volatile int TPMR;
+	volatile int StatusCode;
+	volatile int Part;
 
 	/**
 	 * Multipart messages processing.
